@@ -1,11 +1,11 @@
 # -*- perl -*-
 ##----------------------------------------------------------------------------
 ## Database Object Interface - ~/lib/DB/Object/Postgres/Query.pm
-## Version v0.2.0
+## Version v0.2.1
 ## Copyright(c) 2023 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2017/07/19
-## Modified 2023/11/17
+## Modified 2024/01/05
 ## All rights reserved
 ## 
 ## 
@@ -21,7 +21,7 @@ BEGIN
     use vars qw( $VERSION $DEBUG );
     use Devel::Confess;
     use Want;
-    our $VERSION = 'v0.2.0';
+    our $VERSION = 'v0.2.1';
 };
 
 use strict;
@@ -544,7 +544,6 @@ sub on_conflict
                         # Need to account for placeholders
                         # Let's check values only
                         $self->is_upsert(1);
-                        # my $inherited_fields = $self->format_update( $f_ref );
                         my $elems = $self->format_update( $f_ref );
                         my $inherited_fields = $elems->formats->join( ', ' );
                         push( @comp, 'DO UPDATE SET' );
@@ -552,6 +551,8 @@ sub on_conflict
                         $hash->{query} = join( ' ', @comp );
                         $self->{_on_conflict} = $hash;
                         $self->{on_conflict} = join( ' ', @comp );
+                        $self->elements->push( $elems->elements->list );
+                        $self->messagec( 5, "There are now {green}", $elems->length, "{/} elements for this UPSERT query." );
                         # Usable only once
                         CORE::delete( $self->{_on_conflict_callback} );
                     };
@@ -749,7 +750,7 @@ DB::Object::Postgres::Query - Query Object for PostgreSQL
 
 =head1 VERSION
 
-    v0.2.0
+    v0.2.1
 
 =head1 DESCRIPTION
 
